#include "mbed.h"
#include "SerialGPS.h"

#include "mbed.h"
#include <math.h>
#include "ADXL345_I2C.h"

#define DELTA_SEUIL 50

ADXL345_I2C accelerometer(D14, D15);
Serial pc(USBTX, USBRX); // tx, rx
SerialGPS gps(D8,D2);  // tx, rx




/**
 * A callback function for logging data.
 */
void cbfunc_log(char *s) {

}

/**
 * A callback function for GGA.
 *
 * GGA - Global Positioning System Fixed Data.
 */
void cbfunc_gga(SerialGPS::gps_gga_t *p) {

        pc.printf("%02d:%02d:%02d(P%d,S%d)\r\n", p->hour, p->min, p->sec, p->position_fix, p->satellites_used);

        pc.printf("%c=%10.4f\r\n", p->ns, p->latitude);

        pc.printf("%c=%10.4f\r\n", p->ew, p->longitude);
    
}

/**
 * A callback function for GSA.
 *
 * GSA - GNSS DOP and Active Satellites.
 */
void cbfunc_gsa(SerialGPS::gps_gsa_t *p) {

        pc.printf("SEL:%c FIX:%d\r\n", p->selmode, p->fix);
  
}

/**
 * A callback function for GSV.
 *
 * GSV - GNSS Satellites in View.
 */
void cbfunc_gsv(SerialGPS::gps_gsv_t *p) {
 
        pc.printf("Satellites:%2d\r\n", p->satcnt);

}

/**
 * A callback function for RMC.
 *
 * RMC - Recommended Minimum Specific GNSS Data.
 */
void cbfunc_rmc(SerialGPS::gps_rmc_t *p) {

        pc.printf("%02d:%02d:%02d(%c)\r\n", p->hour, p->min, p->sec, p->status);

}

/**
 * Entry point.
 */

// Accelerometer
int init_accelero(){
     
    printf("Starting ADXL345 test...\n\r");
    wait(.001);
    printf("Device ID is: 0x%02x\n\r", accelerometer.getDeviceID());
    wait(.001);
    
    // These are here to test whether any of the initialization fails. It will print the failure
    if (accelerometer.setPowerControl(0x00))
    {
         printf("didn't intitialize power control\n"); 
         return 0;  
    }
    
    //Full resolution, +/-16g, 4mg/LSB.
    wait(.001);
     
    if(accelerometer.setDataFormatControl(0x0B))
    {
        printf("didn't set data format\n");
        return 0;  
    }
    wait(.001);
     
     //3.2kHz data rate.
    if(accelerometer.setDataRate(ADXL345_3200HZ))
    {
        printf("didn't set data rate\n");
        return 0;    
    }
    wait(.001);
     
    //Measurement mode.     
    if(accelerometer.setPowerControl(MeasurementMode)) 
    {
        printf("didn't set the power control to measurement\n"); 
        return 0;   
    }
    return 1; 
}
void copie_tab(int tab1[], int tab2[], int taille){
    int i;
    for(i = 0; i < taille; i++)
        tab2[i] = tab1[i];
}
bool mouvement (int readings[],int precedent_read[]){
    static int nb_read = 0;
    int i;
    bool mouvement = false;
    accelerometer.getOutput(readings);     
    wait(0.1); // 100 ms
            
    if(nb_read != 0) {
        for(i = 0; i < 3; i++) {
            if(abs(precedent_read[i] - readings[i] > DELTA_SEUIL)) {
                mouvement = true;
            }
            copie_tab(readings,precedent_read,3);
            nb_read++;
        } 
    }
    copie_tab(readings,precedent_read,3);
    nb_read++;
    return mouvement;
}

 
int main() {
    
    pc.baud(9600);
    
    
    pc.printf("TEST\r\n");
    wait(0.5);
    int readings[3] = {0, 0, 0};
    int precedent_read[3] = {0, 0, 0};
    init_accelero();
    
    SerialGPS::gps_callback_t cb;
    cb.cbfunc_log = cbfunc_log;
    cb.cbfunc_gga = cbfunc_gga;
    cb.cbfunc_gsa = cbfunc_gsa;
    cb.cbfunc_gsv = cbfunc_gsv;
    cb.cbfunc_rmc = cbfunc_rmc;
    
    gps.attach(&cb);
    
    while(1) {
       if(mouvement(readings,precedent_read)){
            printf("Mouvement \n\r");
            /* x-axis, y-axis and z-axis */        
            printf("%d, %d, %d\n\r", (int16_t)readings[0], (int16_t)readings[1], (int16_t)readings[2]);
            wait(3);
            
        }
        else {
            //printf("Immobile \n\r"); Cause un bug
            gps.processing();
        }
        //wait(1);

        
    }
}

